@@ -1,15 +1,18 @@
+# Make strings collectable with gettext tools, but don't trnslate them here:
+_ = unicode
+
 # Transaction types (TRXTYPE)...
 SALE, CREDIT, AUTHORIZATION, DELAYED_CAPTURE, VOID, DUPLICATE_TRANSACTION = (
     'S', 'C', 'A', 'D', 'V', 'N')
 
 # ...for humans
 trxtype_map = {
-    SALE: 'Sale',
-    AUTHORIZATION: 'Authorize',
-    CREDIT: 'Credit',
-    DELAYED_CAPTURE: 'Delayed capture',
-    VOID: 'Void',
-    DUPLICATE_TRANSACTION: 'Duplicate transaction',
+    SALE: _('Sale'),
+    AUTHORIZATION: _('Authorize'),
+    CREDIT: _('Credit'),
+    DELAYED_CAPTURE: _('Delayed capture'),
+    VOID: _('Void'),
+    DUPLICATE_TRANSACTION: _('Duplicate transaction'),
 }
 
 # Payment methods (TENDER)
